#include <Arduino.h>
#include <Wire.h>
#include "EEPROM24FC.h"

// UTF16文字コードに対応するFONT PATTERNを読み出す
// フォントデータは8バイト(固定)
boolean  EEPROM24FC::readFont(uint16_t utf16Code,uint8_t *fontData) {
  uint8_t utfH = (utf16Code & 0xFF00) >> 8 ;
  uint8_t utfL = utf16Code & 0xFF ;
  uint16_t fontAddr = 0 ;
  memset(fontData,0,8) ;

  // Indexテーブルが存在するか確認
  uint8_t data = pgm_read_byte(fIndexTable + utfH);
  if (data == 0) return false ;

  // IndexテーブルからFONT PATTERNのアドレスを取得
  uint16_t indexAddr = 512 * data + (uint16_t)utfL * 2  ;
  readMultiData(ROM1,indexAddr,2,(uint8_t *)&fontAddr) ;

  // ---  FONT PATTERN
  readMultiData(ROM2, fontAddr,7,fontData) ;

  return true ;
}

// コレを関数化しないと流石にメンドクサイので切り出しておきます
// I2Cバスが使用可能になるのを最大500m秒待ちます。
// 500m秒待って使用可能にならなかった時のことは考えてません。
void EEPROM24FC::writeReadyCheck(uint8_t i2cAddr) {
  int errCount = 0 ;
  while(errCount < 100) {
    Wire.beginTransmission(i2cAddr);    // I2Cスレーブに対して送信処理を開始する
    int ready = Wire.endTransmission(); // トランザクションが正常に終了する事を持ってI2Cバスが使用可能と判断
    if (ready == 0) {
      break ;
    }
    if (errCount > 100) break ;
    errCount ++ ;
    delay(5) ;
    break;
  }
}

// EEPROMから1バイト読み出し
uint8_t EEPROM24FC::readByteData(int i2cAddr ,uint16_t address ) {
  // 書き込み開始アドレスを上位バイトと下位バイトに分解
  uint8_t adrH = (address & 0xFF00) >> 8 ;
  uint8_t adrL = address & 0xFF ;

  // 書き込み開始アドレス送信
  writeReadyCheck(i2cAddr) ;
  Wire.beginTransmission(i2cAddr);    // I2Cスレーブに対して送信処理を開始する
  Wire.write(adrH);
  Wire.write(adrL);
  Wire.endTransmission();

  int len = Wire.requestFrom(i2cAddr, 1);
  uint8_t data = 0 ;
  if (len == 1) {
    data = Wire.read();
  }
  return data ;
}

// EEPROMからマルチバイト読みだし
void EEPROM24FC::readMultiData(int i2cAddr , uint16_t address , int dataSize , uint8_t* data) {
  // 書き込み開始アドレスを上位バイトと下位バイトに分解
  uint8_t adrH = (address & 0xFF00) >> 8 ;
  uint8_t adrL = address & 0xFF ;
    
  // 書き込み開始アドレス送信
  writeReadyCheck(i2cAddr) ;
  Wire.beginTransmission(i2cAddr);    // I2Cスレーブに対して送信処理を開始する
  Wire.write(adrH);
  Wire.write(adrL);
  Wire.endTransmission();

  int rBytes = dataSize ; // 読み込みデータ長
  // 読み込み開始
  while(rBytes>0) {
    // Wireクラスの制限で一度に読み込めるのは32バイトまでなので
    // 読み込むバイト数-読み込んだバイト数がゼロになるまで繰り返す
    // 書き込み開始アドレスを上位バイトと下位バイトに分解
    uint8_t adrH = (address & 0xFF00) >> 8 ;
    uint8_t adrL = address & 0xFF ;

    int len = Wire.requestFrom(i2cAddr, rBytes);
    for (int j=0;j<len;j++) {
      *data = Wire.read();
      data ++ ;
    }
    rBytes -= len ;
    address += len ;
  }
}
