// EEPROM ADDRESS - 0x50 / 0x54
#ifndef _EEPROM24FC_H_
#define _EEPROM24FC_H_

#define ROM1 0x50       //   0- 512 KByte Block
#define ROM2 0x54       // 512-1024 Kbyte Block

// EEPROM のメモリーマップ
// --- ROM1 :
//    0[0000] ～  255[00FF] : FONT INDEX テーブルの存在フラグ
//  512[0200] ～49663[C1FF] : FONT ADDRESS 2Byte x 256 x 96
//53248[D000] ～53312[D03F] : 半角カナ全角変換テーブル
// --- ROM1
//    0[0000] ～48517[BD85] : フォントデータ (6932文字)

class EEPROM24FC
{
  public:
    // --- ROM ACCESS ---
    void writeReadyCheck(uint8_t i2cAddr) ;   // I2CバスのRady待ち
    uint8_t readByteData(int i2cAddr , uint16_t address) ;  // 1バイト読込み
    void readMultiData(int i2cAddr , uint16_t address , int dataSize , uint8_t* data) ; // マルチバイト読込み

    boolean readFont(uint16_t utfCode,uint8_t *fontData) ; // UTF16コードに対応したフォントパターンを返す
} ;

// ROM 内にもあるけど、Purogram領域に置いた方が圧倒的に速い
// 320Byte位は許容範囲だと思う。
// どうしてもダメならEEPROMから読み込み可
//
// FONT INDEX TABLE
const uint8_t fIndexTable[] PROGMEM = {
   1,   0,   0,   2,   3,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   4,   5,   6,   7,   8,   9,  10,   0,   0,   0,   0,   0,   0,   0,   0,   0,
  11,   0,  12,  13,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,  14,  15,
  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,
  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,
  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,
  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,
  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,  94,  95,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,  96,
};

// 半角カナ全角変換テーブル
const uint8_t hfTable[] PROGMEM = {
0x02,0x0C,0x0D,0x01,0xFB,0xF2,0xA1,0xA3,0xA5,0xA7,0xA9,0xE3,0xE5,0xE7,0xC3,0xFD,
0xA2,0xA4,0xA6,0xA8,0xAA,0xAB,0xAD,0xAF,0xB1,0xB3,0xB5,0xB7,0xB9,0xBB,0xBD,0xBF,
0xC1,0xC4,0xC6,0xC8,0xCA,0xCB,0xCC,0xCD,0xCE,0xCF,0xD2,0xD5,0xD8,0xDB,0xDE,0xDF,
0xE0,0xE1,0xE2,0xE4,0xE6,0xE8,0xE9,0xEA,0xEB,0xEC,0xED,0xEF,0xF3,0x9B,0x9C,0x00,
};

#endif
