#include <Arduino.h>
#include "readFont.h"
#include "EEPROM24FC.h"

EEPROM24FC eeprom24fc ;

// UTF8の文字長(2 or 3)を返す
// ASCIIの場合は1を返す
int READFONT::UTF8LEN(uint8_t *utf8codes) {
  int utfLen = 0 ;
  if( utf8codes[0] < 0x80 ){
    utfLen = 1 ;
  } else
  if( utf8codes[0] < 0xE0 ){
    utfLen = 2 ;
  } else {
    utfLen = 3 ;
  }
  return utfLen ;
}

// UTF8->UTF16
uint16_t READFONT::UTF8ToUTF16(uint8_t *utf8codes) {
  uint16_t utf16Code = 0x0000 ;
  int codeLen = UTF8LEN(utf8codes) ;
 
  if (codeLen == 2) {
    ((uint8_t *)&utf16Code)[0] = ((utf8codes[0] & 0x1F)<<6) + (utf8codes[1] & 0x3F);
    ((uint8_t *)&utf16Code)[1] = ((utf8codes[0] & 0x1F)>>2);
  } else
  if (codeLen == 3) {
    ((uint8_t *)&utf16Code)[1] = ((utf8codes[0] & 0xF)<<4) + ((utf8codes[1] & 0x3F)>>2);
    ((uint8_t *)&utf16Code)[0] = ((utf8codes[1] & 0x3F)<<6) + (utf8codes[2] & 0x3F);
  }
  return utf16Code ;
}

// ----- FONT読みだし
// 半角カナコード判定
boolean  READFONT::isHkana(uint16_t utf16code) {
    return (utf16code >=0xFF61 && utf16code <= 0xFF9F) ? true : false ;
}

// 半角カナ全角変換
uint16_t READFONT::hkana2kana(uint16_t utf16code) {
  if (isHkana(utf16code))
      return pgm_read_byte(hfTable + utf16code - 0xFF61) + 0x3000;
  return utf16code;
}

// UTF16に対応するFONT PATTERN(8Byte)を取得する
boolean  READFONT::getFontDataByUTF16(uint16_t utf16code) {
  return eeprom24fc.readFont(utf16code,fontData) ;
}

// UTF16半角コードをUTF16全角コードに変換する
uint16_t READFONT::utf16_HantoZen(uint16_t utf16code) {
  
  utf16code = hkana2kana(utf16code);

  if (utf16code > 0xff || utf16code < 0x20 ) {
    return utf16code;
  }

  switch(utf16code) {
    case 0x005C:
    case 0x00A2:
    case 0x00A3:
    case 0x00A7:
    case 0x00A8:
    case 0x00AC:
    case 0x00B0:
    case 0x00B1:
    case 0x00B4:
    case 0x00B6:
    case 0x00D7:
    case 0x00F7: return utf16code;
    case 0x00A5: return 0xFFE5;

    case 0x0020: return 0x3000;
    case 0x0021: return 0xFF01;
    case 0x0022: return 0x201D; 
    case 0x0023: return 0xFF03;
    case 0x0024: return 0xFF04;
    case 0x0025: return 0xFF05;
    case 0x0026: return 0xFF06;
    case 0x0027: return 0x2019;
    case 0x0028: return 0xFF08;
    case 0x0029: return 0xFF09;
    case 0x002A: return 0xFF0A;
    case 0x002B: return 0xFF0B;
    case 0x002C: return 0xFF0C;
    case 0x002D: return 0x2212;
    case 0x002E: return 0xFF0E;    
  }  
   return  utf16code - 0x2F +  0xFF0F;     
}
